pub struct AllocationStage<'a, I>where
I: SlabItem,{
pub allocator: &'a mut SlabAllocator<I>,
/* private fields */
}Expand description
An object that allows batched allocation.
In order to perform allocations, you create one of these objects with
SlabAllocator::stage_allocation, allocate into it with
Self::allocate, and finally commit it with Self::commit. Always
make sure to call Self::commit; if you don’t, buffers that were
supposed to be enlarged won’t be.
Fields
allocator: &'a mut SlabAllocator<I>The allocator that we’re allocating objects into.
Implementations
Sourceimpl<'a, I> AllocationStage<'a, I>where
I: SlabItem,
impl<'a, I> AllocationStage<'a, I>where
I: SlabItem,
Sourcepub fn allocate(
&mut self,
key: &I::Key,
data_byte_len: u64,
layout: I::Layout,
settings: &SlabAllocatorSettings,
)
pub fn allocate( &mut self, key: &I::Key, data_byte_len: u64, layout: I::Layout, settings: &SlabAllocatorSettings, )
Allocates space for an object of the given size with the given key and layout.
The key must not correspond to any current allocation.
Sourcepub fn allocate_large(&mut self, key: &I::Key, layout: I::Layout)
pub fn allocate_large(&mut self, key: &I::Key, layout: I::Layout)
Allocates an object into its own dedicated slab.
The key must not correspond to any current allocation.
Sourcepub fn commit(self, render_device: &RenderDevice, render_queue: &RenderQueue)
pub fn commit(self, render_device: &RenderDevice, render_queue: &RenderQueue)
Completes the transaction, performing any queued resize operations.